@@ -1,9 +1,10 @@
-const PluginAuth = require('./plugin-auth');
-const fs = require('fs');
-const crypto = require('crypto');
-const Errors = require('../../../misc/errors');
-const NativePasswordAuth = require('./native-password-auth');
-const Sha256PasswordAuth = require('./sha256-password-auth');
+//  SPDX-License-Identifier: LGPL-2.1-or-later
+//  Copyright (c) 2015-2025 MariaDB Corporation Ab
+
+import PluginAuth from './plugin-auth.js';
+import fs from 'node:fs';
+import * as Errors from '../../../misc/errors.js';
+import Sha256PasswordAuth from './sha256-password-auth.js';
 
 const State = {
   INIT: 'INIT',
@@ -16,28 +17,34 @@ const State = {
  * Use caching Sha2 password authentication
  */
 class CachingSha2PasswordAuth extends PluginAuth {
-  constructor(packSeq, compressPackSeq, pluginData, resolve, reject, multiAuthResolver) {
-    super(resolve, reject, multiAuthResolver);
+  constructor(packSeq, compressPackSeq, pluginData, cmdParam, reject, multiAuthResolver) {
+    super(cmdParam, multiAuthResolver, reject);
+    this.multiAuthResolver = multiAuthResolver;
     this.pluginData = pluginData;
     this.sequenceNo = packSeq;
+    this.compressSequenceNo = compressPackSeq;
     this.counter = 0;
     this.state = State.INIT;
   }
 
+  isSecureConnection() {
+    return this.cmdParam.opts.ssl || (this.cmdParam.opts.socketPath && process.platform !== 'win32');
+  }
+
   start(out, opts, info) {
     this.exchange(this.pluginData, out, opts, info);
     this.onPacketReceive = this.response;
   }
 
-  exchange(buffer, out, opts, info) {
+  exchange(packet, out, opts, info) {
     switch (this.state) {
       case State.INIT:
         const truncatedSeed = this.pluginData.slice(0, this.pluginData.length - 1);
-        const encPwd = NativePasswordAuth.encryptPassword(opts.password, truncatedSeed, 'sha256');
+        const encPwd = Sha256PasswordAuth.encryptSha256Password(opts.password, truncatedSeed);
         out.startPacket(this);
         if (encPwd.length > 0) {
           out.writeBuffer(encPwd, 0, encPwd.length);
-          out.flushBuffer(true);
+          out.flushPacket();
         } else {
           out.writeEmptyPacket(true);
         }
@@ -46,20 +53,31 @@ class CachingSha2PasswordAuth extends PluginAuth {
 
       case State.FAST_AUTH_RESULT:
         // length encoded numeric : 0x01 0x03/0x04
-        const fastAuthResult = buffer[1];
+        const fastAuthResult = packet[1];
         switch (fastAuthResult) {
           case 0x03:
             // success authentication
-            this.emit('send_end');
-            return this.successSend(packet, out, opts, info);
+            // an OK_Packet will follow
+            return;
 
           case 0x04:
-            if (opts.ssl) {
-              // using SSL, so sending password in clear
+            if (this.isSecureConnection()) {
+              if (info.requireValidCert && info.selfSignedCertificate) {
+                return this.throwError(
+                  Errors.createFatalError(
+                    'cannot send a `caching_sha2_password` and SSL without providing server certificates,' +
+                      ' cannot send password in clear on an untrusted SSL connection',
+                    Errors.client.ER_SELF_SIGNED_SHA256,
+                    info
+                  ),
+                  info
+                );
+              }
+              // Secure connection, sending password in clear
               out.startPacket(this);
               out.writeString(opts.password);
               out.writeInt8(0);
-              out.flushBuffer(true);
+              out.flushPacket();
               return;
             }
 
@@ -71,29 +89,20 @@ class CachingSha2PasswordAuth extends PluginAuth {
                   // rsaPublicKey contain path
                   key = fs.readFileSync(key, 'utf8');
                 }
-                this.publicKey = Sha256PasswordAuth.retreivePublicKey(key);
+                this.publicKey = Sha256PasswordAuth.retrievePublicKey(key);
               } catch (err) {
                 return this.throwError(err, info);
               }
               // send Sha256Password Packet
-              Sha256PasswordAuth.sendSha256PwdPacket(
-                this,
-                this.pluginData,
-                this.publicKey,
-                opts.password,
-                out
-              );
+              Sha256PasswordAuth.sendSha256PwdPacket(this, this.pluginData, this.publicKey, opts.password, out);
             } else {
               if (!opts.allowPublicKeyRetrieval) {
                 return this.throwError(
-                  Errors.createError(
+                  Errors.createFatalError(
                     'RSA public key is not available client side. Either set option `cachingRsaPublicKey` to indicate' +
                       ' public key path, or allow public key retrieval with option `allowPublicKeyRetrieval`',
-                    null,
-                    true,
-                    info,
-                    '08S01',
-                    Errors.ER_CANNOT_RETRIEVE_RSA_KEY
+                    Errors.client.ER_CANNOT_RETRIEVE_RSA_KEY,
+                    info
                   ),
                   info
                 );
@@ -102,50 +111,19 @@ class CachingSha2PasswordAuth extends PluginAuth {
               // ask caching public Key Retrieval
               out.startPacket(this);
               out.writeInt8(0x02);
-              out.flushBuffer(true);
+              out.flushPacket();
             }
-            return;
         }
+        return;
 
       case State.REQUEST_SERVER_KEY:
-        this.publicKey = Sha256PasswordAuth.retreivePublicKey(buffer.toString('utf8', 1));
+        this.publicKey = Sha256PasswordAuth.retrievePublicKey(packet.toString(undefined, 1));
         this.state = State.SEND_AUTH;
-        Sha256PasswordAuth.sendSha256PwdPacket(
-          this,
-          this.pluginData,
-          this.publicKey,
-          opts.password,
-          out
-        );
+        Sha256PasswordAuth.sendSha256PwdPacket(this, this.pluginData, this.publicKey, opts.password, out);
+        return;
     }
   }
 
-  static retreivePublicKey(key) {
-    return key.replace('(-+BEGIN PUBLIC KEY-+\\r?\\n|\\n?-+END PUBLIC KEY-+\\r?\\n?)', '');
-  }
-
-  static sendSha256PwdPacket(cmd, pluginData, publicKey, password, out) {
-    const truncatedSeed = pluginData.slice(0, pluginData.length - 1);
-    out.startPacket(cmd);
-    const enc = Sha256PasswordAuth.encrypt(truncatedSeed, password, publicKey);
-    out.writeBuffer(enc, 0, enc.length);
-    out.flushBuffer(cmd);
-  }
-
-  // encrypt password with public key
-  static encrypt(seed, password, publicKey) {
-    const nullFinishedPwd = Buffer.from(password + '\0');
-    const xorBytes = Buffer.allocUnsafe(nullFinishedPwd.length);
-    const seedLength = seed.length;
-    for (let i = 0; i < xorBytes.length; i++) {
-      xorBytes[i] = nullFinishedPwd[i] ^ seed[i % seedLength];
-    }
-    return crypto.publicEncrypt(
-      { key: publicKey, padding: crypto.constants.RSA_PKCS1_OAEP_PADDING },
-      xorBytes
-    );
-  }
-
   response(packet, out, opts, info) {
     const marker = packet.peek();
     switch (marker) {
@@ -155,7 +133,7 @@ class CachingSha2PasswordAuth extends PluginAuth {
       case 0x00:
       case 0xff:
         this.emit('send_end');
-        return this.successSend(packet, out, opts, info);
+        return this.multiAuthResolver(packet, out, opts, info);
 
       default:
         let promptData = packet.readBufferRemaining();
@@ -165,4 +143,4 @@ class CachingSha2PasswordAuth extends PluginAuth {
   }
 }
 
-module.exports = CachingSha2PasswordAuth;
+export default CachingSha2PasswordAuth;
